@@ -3,6 +3,8 @@
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,6 +17,13 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
 #include "zhttprequest.h"
@@ -96,6 +105,7 @@ public:
 	QTimer *expireTimer;
 	QTimer *keepAliveTimer;
 	bool multi;
+	bool quiet;
 
 	Private(ZhttpRequest *_q) :
 		QObject(_q),
@@ -123,7 +133,8 @@ public:
 		errored(false),
 		expireTimer(0),
 		keepAliveTimer(0),
-		multi(false)
+		multi(false),
+		quiet(false)
 	{
 		expireTimer = new QTimer(this);
 		connect(expireTimer, &QTimer::timeout, this, &Private::expire_timeout);
@@ -939,6 +950,8 @@ public slots:
 						p.ignoreTlsErrors = true;
 					if(passthrough.isValid())
 						p.passthrough = passthrough;
+					if(quiet)
+						p.quiet = true;
 					writePacket(p);
 
 					state = ClientRequestFinishWait;
@@ -987,6 +1000,8 @@ public slots:
 					p.ignoreTlsErrors = true;
 				if(passthrough.isValid())
 					p.passthrough = passthrough;
+				if(quiet)
+					p.quiet = true;
 				p.credits = IDEAL_CREDITS;
 				p.multi = true;
 				writePacket(p);
@@ -1170,6 +1185,11 @@ void ZhttpRequest::setPassthroughData(const QVariant &data)
 	d->passthrough = data;
 }
 
+void ZhttpRequest::setQuiet(bool on)
+{
+	d->quiet = on;
+}
+
 void ZhttpRequest::start(const QString &method, const QUrl &uri, const HttpHeaders &headers)
 {
 	assert(!d->server);
